import 'package:flutter/material.dart';
import 'package:flutterx_material_tool/flutterx_material_tool.dart';

void main() => runApp(const MyApp());

class MyApp extends StatelessWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) => MaterialApp(
      debugShowCheckedModeBanner: false,
      title: 'Flutterx Material Tool Demo',
      theme: ThemeData(primarySwatch: Colors.grey),
      home: const LoggableExample());
}

class LoggableExample extends StatefulWidget {
  const LoggableExample({Key? key}) : super(key: key);

  @override
  State<LoggableExample> createState() => _LoggableExampleState();
}

class _LoggableExampleState extends State<LoggableExample> {
  final RGBColor _color = Colors.pink.rgb;
  final MaterialPalette<ColorSwatch<int>> _palette = MaterialPalette.primary;

  @override
  Widget build(BuildContext context) => Scaffold(
      appBar: AppBar(title: const Text('Material Tool example')),
      body: Center(
        child: Column(mainAxisAlignment: MainAxisAlignment.center, children: [
          _buildPalette(_palette.generate(_color).data),
          _buildPalette(_palette.generateComplementary(_color).data),
          _buildPalette(_palette.generateAnalogousA(_color).data),
          _buildPalette(_palette.generateAnalogousB(_color).data),
          _buildPalette(_palette.generateTriadicA(_color).data),
          _buildPalette(_palette.generateTriadicB(_color).data),
        ]),
      ));

  Widget _buildPalette(ColorSwatch<int> swatch) {
    final size = MediaQuery.of(context).size.shortestSide / (_palette.shades.length + .8);
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: _palette.shades
          .map((shade) => Container(height: size, width: size, margin: const EdgeInsets.all(1), color: swatch[shade]))
          .toList(growable: false)
          .reversed
          .toList(growable: false),
    );
  }
}
