import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutterx_material_tool/flutterx_material_tool.dart';
import 'package:flutterx_material_tool/src/model/colors.dart';
import 'package:flutterx_material_tool/src/model/material_palette.dart';
import 'package:flutterx_material_tool/src/tool.dart';

@immutable
class PaletteData<T> {
  final MaterialPalette<T> palette;
  final RGBColor input;
  final int inputIndex;
  final List<Color> swatch;

  const PaletteData(this.palette, this.input, this.inputIndex, this.swatch)
      : assert(inputIndex >= 0 && inputIndex < swatch.length);

  factory PaletteData.fromColor(RGBColor input, MaterialPalette<T> palette) {
    final swatch = generateSwatch(input, palette).map((rgb) => rgb.toColor()).toList(growable: false);
    return PaletteData(palette, input, swatch.indexOf(input.toColor()), swatch);
  }

  T get data => palette.map(this);

  Color operator [](int index) => swatch[index];

  Color operator -(int delta) => swatch[(inputIndex - delta).clamp(0, swatch.length - 1)];

  Color operator +(int delta) => swatch[(inputIndex + delta).clamp(0, swatch.length - 1)];

  PaletteData get complementary => palette.generateComplementary(input);

  PaletteData get analogousB => palette.generateAnalogousA(input);

  PaletteData get analogousA => palette.generateAnalogousB(input);

  PaletteData get triadicA => palette.generateTriadicA(input);

  PaletteData get triadicB => palette.generateTriadicB(input);
}
